@@ -28,7 +28,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 
 object Vignore_maximum_pages;
-
+void call_after_gbc_hook();
 
 #include "page.h"
 
@@ -170,7 +170,7 @@ add_page_to_freelist(p,tm)
  tm->tm_npage++;
 }
 
-
+#define PERCENT_FREE(tm)  ((tm->tm_percent_free ? tm->tm_percent_free : 10)/100.0)
 
 object
 alloc_object(t)
@@ -218,8 +218,9 @@ ONCE_MORE:
 CALL_GBC:
 	GBC(tm->tm_type);
 	if (tm->tm_nfree == 0 ||
-	    (float)tm->tm_nfree * 10.0 < (float) TOTAL_THIS_TYPE(tm))
+	    ((float)tm->tm_nfree)  <   (PERCENT_FREE(tm) * TOTAL_THIS_TYPE(tm)))
 		goto EXHAUSTED;
+	call_after_gbc_hook(t);
 	goto ONCE_MORE;
 
 EXHAUSTED:
@@ -228,6 +229,7 @@ EXHAUSTED:
 			tm->tm_maxpage += 1;
 		else
 			tm->tm_maxpage += tm->tm_maxpage/2;
+		call_after_gbc_hook(t);
 		goto ONCE_MORE;
 	}
 	GBC_enable = FALSE;
@@ -241,6 +243,7 @@ Use ALLOCATE to expand the space.",
 		2, vs_top[-2], vs_top[-1]);
 	vs_pop;
 	vs_pop;
+	call_after_gbc_hook(t);
 	goto ONCE_MORE;
 }
 
@@ -302,8 +305,9 @@ ONCE_MORE:
 CALL_GBC:
 	GBC(t_cons);
 	if (tm->tm_nfree == 0 ||
-	    (float)tm->tm_nfree * 10.0 < (float) TOTAL_THIS_TYPE(tm))
+	    (float)tm->tm_nfree   <  PERCENT_FREE(tm) * TOTAL_THIS_TYPE(tm))
 		goto EXHAUSTED;
+	call_after_gbc_hook(t_cons);
 	goto ONCE_MORE;
 
 EXHAUSTED:
@@ -311,7 +315,8 @@ EXHAUSTED:
 	  tm->tm_maxpage =
 	    grow_linear(tm->tm_maxpage,tm->tm_growth_percent,
 			tm->tm_min_grow,tm->tm_max_grow);
-		goto ONCE_MORE;
+	  call_after_gbc_hook(t_cons);
+	  goto ONCE_MORE;
 	}
 	GBC_enable = FALSE;
 	vs_push(make_fixnum(tm->tm_npage));
@@ -322,6 +327,7 @@ Use ALLOCATE to expand the space.",
 		"Continues execution.",
 		1, vs_top[-1]);
 	vs_pop;
+	call_after_gbc_hook(t_cons);
 	goto ONCE_MORE;
 #undef	tm
 }
@@ -338,6 +344,41 @@ object on_stack_cons(x,y)
 }
 
 
+static object Vafter_gbc_hook;
+void
+call_after_gbc_hook(t)
+{ if (Vafter_gbc_hook && Vafter_gbc_hook->s.s_dbind!= Cnil)
+    { set_up_string_register(tm_table[(int)t].tm_name+1);
+      ifuncall1(Vafter_gbc_hook->s.s_dbind,intern(string_register,system_package));
+    }
+}
+
+
+object
+fSallocated(typ)
+     object typ;
+{ struct typemanager *tm=(&tm_table[t_from_type(typ)]);
+  tm = & tm_table[tm->tm_type];
+  if (tm->tm_type == t_relocatable)
+    return list(5,make_fixnum((rb_pointer - rb_start)),
+		make_fixnum((rb_limit - rb_pointer)),
+		make_fixnum(tm->tm_npage),
+		make_fixnum(tm->tm_maxpage),
+		make_fixnum(tm->tm_gbccount));
+  else if (tm->tm_type == t_contiguous)
+    return list(5,Cnil,
+		  Cnil,
+		  make_fixnum(tm->tm_npage),
+		  make_fixnum(tm->tm_maxpage),
+		  make_fixnum(tm->tm_gbccount));
+  else
+    return list(5,make_fixnum(tm->tm_nused),
+	      make_fixnum(tm->tm_nfree),
+	      make_fixnum(tm->tm_npage),
+	      make_fixnum(tm->tm_maxpage),
+	      make_fixnum(tm->tm_gbccount)
+	      );
+}
  
 
 #define	round_up(n)	(((n) + 03) & ~03)
@@ -384,6 +425,7 @@ ONCE_MORE:
 		if (!g) {
 			GBC(t_contiguous);
 			g = TRUE;
+			call_after_gbc_hook(t_contiguous);
 			goto ONCE_MORE;
 		}
 		if (symbol_value(Vignore_maximum_pages) != Cnil)
@@ -391,6 +433,7 @@ ONCE_MORE:
 		   maxcbpage=grow_linear(maxcbpage,tm->tm_growth_percent,
 			      tm->tm_min_grow, tm->tm_max_grow);
 			g = FALSE;
+			call_after_gbc_hook(t_contiguous);
 			goto ONCE_MORE;
 		}
 		vs_push(make_fixnum(ncbpage));
@@ -400,6 +443,7 @@ Use ALLOCATE-CONTIGUOUS-PAGES to expand the space.",
 			"Continues execution.", 1, vs_head);
 		vs_pop;
 		g = FALSE;
+		call_after_gbc_hook(t_contiguous);
 		goto ONCE_MORE;
 	}
 
@@ -464,11 +508,12 @@ ONCE_MORE:
 			{ float f1 = (float)(rb_limit - rb_pointer),
 				f2 = (float)(rb_limit - rb_start);
 
-				if (f1 * 10.0 < f2) 
+				if ((float)f1 < PERCENT_FREE(tm_of(t_relocatable)) * f2) 
 				;
 			else
+			  {	call_after_gbc_hook(t_relocatable);
 				goto ONCE_MORE;
-			}
+					      }}
 		}
 		if (symbol_value(Vignore_maximum_pages) != Cnil)
 		  {struct typemanager *tm = &tm_table[(int)t_relocatable];
@@ -484,6 +529,7 @@ ONCE_MORE:
 			    FEerror("bad rb_end");
 			  alloc_page(-( nrbpage + holepage));
 			  g = FALSE;
+			  call_after_gbc_hook(t_relocatable);
 			  goto ONCE_MORE;
 			}
 		}
@@ -497,6 +543,7 @@ Use ALLOCATE-RELOCATABLE-PAGES to expand the space.",
 			"Continues execution.", 1, vs_head);
 		vs_pop;
 		g = FALSE;
+		call_after_gbc_hook(t_relocatable);
 		goto ONCE_MORE;
 	}
 	p = rb_pointer;
@@ -702,7 +749,7 @@ t_from_type(type)
 {object typ= coerce_to_string(type);
  object c = aref1(typ,0);
  int i;
- for (i= (int)t_start ; i < (int)t_contiguous ; i++)
+ for (i= (int)t_start ; i < (int)t_other ; i++)
    {struct typemanager *tm = &tm_table[i];
    if(tm->tm_name &&
       0==strncmp((tm->tm_name)+1,typ->st.st_self,typ->st.st_fillp)
@@ -740,21 +787,25 @@ siSallocate_sgc(type,min,max,free_percent)
    It will try to grow PERCENT of the current pages.
    */
 object
-siSallocate_growth(type,min,max,percent)
+siSallocate_growth(type,min,max,percent,percent_free)
 int min,max,percent;
 object type;
 {int  t=t_from_type(type);
  struct typemanager *tm=tm_of(t);
  object res;
- res= list(3,make_fixnum(tm->tm_min_grow),
+ res= list(4,make_fixnum(tm->tm_min_grow),
 	   make_fixnum(tm->tm_max_grow),
-	   make_fixnum(tm->tm_growth_percent));
+	   make_fixnum(tm->tm_growth_percent),
+	   make_fixnum(tm->tm_percent_free));
  
- if(min<0 || max< min || min > 3000 || percent < 0 || percent > 500)
+ if(min<0 || max< min || min > 3000 || percent < 0 || percent > 500 
+    || percent_free <0 || percent_free > 100
+    )
     goto END;
  tm->tm_max_grow=max;
  tm->tm_min_grow=min;
  tm->tm_growth_percent= percent;
+ tm->tm_percent_free= percent_free;
  END:
  return res;
 }
@@ -892,15 +943,19 @@ init_alloc_function()
 		     4 | ARGTYPE(0,f_object) | ARGTYPE(1,f_fixnum) |
 		     ARGTYPE(2,f_fixnum) | ARGTYPE(3,f_fixnum)
 		     | RESTYPE(f_object));
-
-
-	make_si_sfun("ALLOCATE-GROWTH",siSallocate_growth,
-		     4 | ARGTYPE(0,f_object) | ARGTYPE(1,f_fixnum) |
-		     ARGTYPE(2,f_fixnum) | ARGTYPE(3,f_fixnum)
+	make_si_sfun("ALLOCATED",fSallocated,
+		     1 | ARGTYPE(0,f_object) 
 		     | RESTYPE(f_object));
+	make_si_sfun("ALLOCATE-GROWTH",siSallocate_growth,
+		     5 | ARGTYPE(0,f_object) | ARGTYPE(1,f_fixnum) |
+		     ARGTYPE(2,f_fixnum) | ARGTYPE(3,f_fixnum) | ARGTYPE(4,f_fixnum)
+		     | RESTYPE(f_object));
+	
+        Vafter_gbc_hook = make_si_special("*AFTER-GBC-HOOK*",Cnil);
 	Vignore_maximum_pages
 	= make_special("*IGNORE-MAXIMUM-PAGES*", Ct);
 
+
 }
 
 object malloc_list;
