@@ -925,12 +925,7 @@ enum type t;
 
 	what_to_collect = t;
 
-	if (t == t_contiguous)
-		cbgbccount++;
-	else if (t == t_relocatable)
-		rbgbccount++;
-	else
-		tm_table[(int)t].tm_gbccount++;
+	tm_table[(int)t].tm_gbccount++;
 
 #ifdef DEBUG
 	if (debug || (siVnotify_gbc->s.s_dbind != Cnil)) {
@@ -1218,9 +1213,8 @@ siLreset_gbc_count()
 	int i;
 
 	check_arg(0);
-	cbgbccount = 0;
-	rbgbccount = 0;
-	for (i = 0;  i < (int)t_end;  i++)
+
+	for (i = 0;  i < (int)t_other;  i++)
 		tm_table[i].tm_gbccount = 0;
 }
 
