@@ -91,7 +91,9 @@ int cssize;
 int sgc_enabled;
 void install_segmentation_catcher();
 
+#ifndef SIG_STACK_SIZE
 #define SIG_STACK_SIZE 1000
+#endif
 #ifndef SETUP_SIG_STACK
 #if defined(HAVE_SIGACTION) || defined(HAVE_SIGVEC)
         struct sigstack estack;
@@ -565,7 +567,7 @@ vs_overflow()
 {
 	if (vs_limit > vs_org + stack_multiple *  VSSIZE)
 		error("value stack overflow");
-	vs_limit += VSGETA;
+	vs_limit += STACK_OVER*VSGETA;
 	FEerror("Value stack overflow.", 0);
 }
 
@@ -575,7 +577,7 @@ bds_overflow()
 	--bds_top;
 	if (bds_limit > bds_org + stack_multiple *  BDSSIZE)
 		error("bind stack overflow");
-	bds_limit += BDSGETA;
+	bds_limit += STACK_OVER  *BDSGETA;
 	FEerror("Bind stack overflow.", 0);
 }
 
@@ -584,7 +586,7 @@ frs_overflow()
 	--frs_top;
 	if (frs_limit > frs_org + stack_multiple *  FRSSIZE)
 		error("frame stack overflow");
-	frs_limit += FRSGETA;
+	frs_limit += STACK_OVER* FRSGETA;
 	FEerror("Frame stack overflow.", 0);
 }
 
@@ -593,7 +595,7 @@ ihs_overflow()
 	--ihs_top;
 	if (ihs_limit > ihs_org + stack_multiple *  IHSSIZE)
 		error("invocation history stack overflow");
-	ihs_limit += IHSGETA;
+	ihs_limit += STACK_OVER*IHSGETA;
 	FEerror("Invocation history stack overflow.", 0);
 }
 
@@ -800,8 +802,8 @@ siLreset_stack_limits(arg)
   bcopy(org,p,leng=(stack_multiple*size*sizeof(typ))); \
   topl= top - org; \
   org=(typ *)p; top = org +topl;\
-  p=p+leng+2*geta*sizeof(typ); \
-  lim = ((typ *)p) - 2*geta;   \
+  p=p+leng+(STACK_OVER+1)*geta*sizeof(typ); \
+  lim = ((typ *)p) - (STACK_OVER+1)*geta;   \
   }while (0)
 
 multiply_stacks(m)
@@ -813,10 +815,10 @@ multiply_stacks(m)
    int vs,bd,frs,ihs;
    stack_multiple=stack_multiple*m;
 #define ELTSIZE(x) (((char *)((x)+1)) - ((char *) x))
-   vs  = (stack_multiple*VSSIZE  + 2*VSGETA)* ELTSIZE(vs_org);
-   bd  = (stack_multiple*BDSSIZE + 2*BDSGETA)*ELTSIZE(bds_org);
-   frs = (stack_multiple*FRSSIZE + 2*FRSGETA)*ELTSIZE(frs_org);
-   ihs = (stack_multiple*IHSSIZE + 2*IHSGETA)*ELTSIZE(ihs_org);
+   vs  = (stack_multiple*VSSIZE  + (STACK_OVER+1)*VSGETA)* ELTSIZE(vs_org);
+   bd  = (stack_multiple*BDSSIZE + (STACK_OVER+1)*BDSGETA)*ELTSIZE(bds_org);
+   frs = (stack_multiple*FRSSIZE + (STACK_OVER+1)*FRSGETA)*ELTSIZE(frs_org);
+   ihs = (stack_multiple*IHSSIZE + (STACK_OVER+1)*IHSGETA)*ELTSIZE(ihs_org);
    if (stack_space==0) {enter_mark_origin(&stack_space);}
    stack_space = alloc_simple_string(vs+bd+frs+ihs);
    array_allocself(stack_space,1,code_char(0));
