@@ -1,13 +1,4 @@
 
-
-#ifndef __GNUC__
-#define HAVE_ALLOCA
-#include <alloca.h>
-#endif
- /* If can mprotect pages and so selective gc will work */
-#define SGC  
-
-/* End for cmpinclude */
 /* Begin for cmpinclude */
 #ifndef __GNUC__
 #define HAVE_ALLOCA
@@ -17,37 +8,6 @@
 #define SGC  
 
 /* End for cmpinclude */
-/* Begin for cmpinclude */
-#ifndef __GNUC__
-#define HAVE_ALLOCA
-#include <alloca.h>
-#endif
- /* If can mprotect pages and so selective gc will work */
-#define SGC  
-
-/* Begin for cmpinclude */
-#ifndef __GNUC__
-#define HAVE_ALLOCA
-#include <alloca.h>
-#endif
- /* If can mprotect pages and so selective gc will work */
-#define SGC  
-
-/* Begin for cmpinclude */
-#ifndef __GNUC__
-#define HAVE_ALLOCA
-#include <alloca.h>
-#endif
- /* If can mprotect pages and so selective gc will work */
-#define SGC  
-
-/* Begin for cmpinclude */
-#ifndef __GNUC__
-#define HAVE_ALLOCA
-#include <alloca.h>
-#endif
- /* If can mprotect pages and so selective gc will work */
-#define SGC  
 
 
 /*
@@ -1243,7 +1203,7 @@ double object_to_double();
 char *object_to_string();
 int FIXtemp;
 #define	CMPmake_fixnum(x) \
-((((FIXtemp=(x))+1024)&-2048)==0?small_fixnum(FIXtemp):make_fixnum(FIXtemp))
+((((FIXtemp=(x))+SMALL_FIXNUM_LIMIT)&(-2*SMALL_FIXNUM_LIMIT))==0?small_fixnum(FIXtemp):make_fixnum(FIXtemp))
 #define Creturn(v) return((vs_top=vs,(v)))
 #define Cexit return((vs_top=vs,0))
 double sin(), cos(), tan();
