@@ -708,9 +708,11 @@ struct typemanager {
 	short   tm_min_grow;    /* min amount to grow when growing */
 	short   tm_max_grow;    /* max amount to grow when growing */
 	short   tm_growth_percent;  /* percent to increase maxpages */
+	short   tm_percent_free;  /* percent which must be free after a gc for this type */
 
 };
 
+
 /*
 	The table of type managers.
 */
@@ -736,13 +738,20 @@ struct contblock *cb_pointer;	/*  contblock pointer  */
 	Variables for memory management.
 */
 int ncb;			/*  number of contblocks  */
-int ncbpage;			/*  number of contblock pages  */
-int maxcbpage;			/*  maximum number of contblock pages  */
-int cbgbccount;			/*  contblock gbc count  */
+/* int ncbpage;			  number of contblock pages  */
+#define ncbpage tm_table[t_contiguous].tm_npage
+#define maxcbpage   tm_table[t_contiguous].tm_maxpage
+#define cbgbccount tm_table[t_relocatable].tm_gbccount  
+  
+
+/* int maxcbpage; maximum number of contblock pages  */
 
 int holepage;			/*  hole pages  */
-int nrbpage;			/*  number of relblock pages  */
-int rbgbccount;			/*  relblock gbc count  */
+#define nrbpage tm_table[t_relocatable].tm_npage
+#define rbgbccount tm_table[t_relocatable].tm_gbccount
+/* int nrbpage;			  number of relblock pages  */
+  
+
 
 char *rb_start;			/*  relblock start  */
 char *rb_end;			/*  relblock end  */
